#!/bin/sh

prefix=/usr/bin

# Modify encoding properties here:
srcfrm=zmplayer
pass=2
profile=normal
quality=FAST

# Global options
#gopts="-quiet"
gopts=""
gvopts="-sws 9 -ffourcc DX50 -noodml"

# Subtitle options
sopts=""
#[ -e ${fn}.srt ] && sopts="-subfont-autoscale 3 -subcp latin1 -font Arial.ttf -sub ${fn}.srt"
#[ -e ${fn}.sub ] && sopts="-subfont-autoscale 3 -subcp latin1 -font Arial.ttf -sub ${fn}.sub"

idmedia() {
  mplayer -identify -frames 0 -ao null -vo null -vc dummy "$@" 2> /dev/null | grep ID_
}
disp() {
  echo "$@" >&1
}
warn() {
  echo "WARNING: $@" >&2
}
die() {
  echo "FAILURE: $@" >&2
  exit 1
}

if [ -f "$1" ];
then
  fn=`basename $1 .avi`
  ifn=${fn}.avi
  ofn=${fn}-new.avi
else
  die "File $1 not found!"
fi

# Identification of the file format (16:9, 4:3, fpѕ etc.)
vw=`idmedia "$ifn" | grep ID_VIDEO_WIDTH  | cut -f2 -d=`
vh=`idmedia "$ifn" | grep ID_VIDEO_HEIGHT  | cut -f2 -d=`
fps=`idmedia "$ifn" | grep ID_VIDEO_FPS | cut -f2 -d=`
is43=$(((vw * 3 * 100) / (vh * 4) - 100))
is169=$(((vw * 9 * 100) / (vh * 16) - 100))
# the closest to 0 is the winner...
[ $is43 -lt 0 ] && is43=$((- is43))
[ $is169 -lt 0 ] && is169=$((- is169))
aspr="16:9"
[ $is43 -lt $is169 ] && aspr="4:3"

disp "File: ${fn}.avi video is ${vw}x${vh} with $fps fps and the aspect ratio is thus $aspr"

# quality profile according to mplayer documentation
case "$quality" in
  VHQ)
    vopts="lavc -lavcopts vcodec=mpeg4:mbd=2:mv0:trell:v4mv:cbp:last_pred=3:predia=2:dia=2:vmax_b_frames=2:vb_strategy=1:precmp=2:cmp=2:subcmp=2:preme=2:qns=2:vbitrate=";;
  HQ)
    vopts="lavc -lavcopts vcodec=mpeg4:mbd=2:trell:v4mv:last_pred=2:dia=-1:vmax_b_frames=2:vb_strategy=1:cmp=3:subcmp=3:precmp=0:vqcomp=0.6:turbo:vbitrate=";;
  FAST)
    vopts="lavc -lavcopts vcodec=mpeg4:mbd=2:trell:v4mv:turbo:vbitrate=";;
  RT)
    vopts="lavc -lavcopts vcodec=mpeg4:mbd=2:turbo:vbitrate=";;
esac

case "$profile" in
  normal)
    gaopts="-srate 22050"
# mode= stereo 0, joint stereo 1, dualchannel 2, mono 3
    aopts="mp3lame -lameopts mode=2:abr:br="
    case "$aspr" in
      4:3) vbr=200; abr=64; scale=320:240;;
      16:9) vbr=200; abr=64; scale=320:176;;
    esac;;
  gprs)
# reduce data rate: enforce mono and sampling rate of 8k plus 15 fps output frame rate
    gaopts="-srate 8000"
    aopts="mp3lame -lameopts mode=3:abr:br="
    gvopts="${gvopts} -ofps 15"
    case "$aspr" in
      4:3) vbr=64; abr=24; scale=320:240;;
      16:9) vbr=64; abr=24; scale=320:176;;
    esac;;
esac
# scaling options here we rotate: remove it if you don't want it
vf="scale=${scale},rotate=1"

disp "Encoding parameters video: ${vopts}${vbr} and audio: ${aopts}${abr}"

case "$pass" in
  "1") 
    $prefix/mencoder "${ifn}" -o "${ofn}" $gopts \
      $gaopts -oac ${aopts}${abr} \
      $gvopts $sopts -vf $vf -ovc ${vopts}${vbr}
    ;;
  "2") 
    rm -f frameno.avi divx2pass.log
    $prefix/mencoder "${ifn}" -o /dev/null $gopts \
      -info srcform="$srcfrm" \
      -nosound \
      $gvopts $sopts -vf $vf -ovc ${vopts}${vbr}:vpass=1
    $prefix/mencoder "${ifn}" -o "${ofn}" $gopts \
      -info srcform="$srcfrm" \
      $gaopts -oac ${aopts}${abr} \
      $gvopts $sopts -vf $vf -ovc ${vopts}${vbr}:vpass=2
    rm -f frameno.avi divx2pass.log
    ;;
esac
